      program ptspec
c
c     this program provides perodgrams of point process data with
c     the significant bands ( 0.90, 0.95 and 0.99 ) assuming the
c     stationary poisson process.   powers of interested frequencies
c     or periods are plotted by a particular sign.
c
c     structure of the program
c
c          ptspec
c             i---input
c             i---cycle
c             i---period
c             i---printr
c             +---smooth
c
c     this is designed by y. ogata and t. ozaki, and programmed by
c     k. katsura and t. ozaki, inst. statist. math., tokyo. (31/01/85)
c
c     reference
c     d. vere-jones and t. ozaki (1982). "some examples of statistical
c        estimation applied to earthquake data, 1. cyclic poisson and
c        self-exciting models."  ann. inst. statist. math., vol. 34,
c        no. 1, pp. 189-207.
c
      implicit real*8 (a-h,o-z)
      dimension t(5000)
      dimension h(5000),g(5000),s(5000)
      dimension w(5000)
      dimension tmpr(100),wt(200),ht(200),gt(200)
      real*4 widthx,widthy
      call input(t,n,t0,pi2,rpt,tmpr,tmp,prd,nh1,nt,is,ipl)
      call cycle(t,n,prd)
      call period(h,g,w,n,t,nh1,ht,gt,wt,nt,rpt,t0,pi2,tmpr)
      if(is.gt.1) go to 10
c
      isw1=1
      call printr(nh1,w,h,g,nt,wt,ht,t0,tmp,isw1)
c
      go to 20
   10 call smooth(s,h,nh1,is)
      isw1=2
      call printr(nh1,w,s,g,nt,wt,ht,t0,tmp,isw1)
   20 continue
      stop
      end
      subroutine input(xx,nn,t,pi2,rpt,tmpr,tmp,prd,nh1,nt,is,ipl)
c
c     inputs
c
c     nh:  number of sampling frequencies of spectra
c     tmp:  the minimum periodicity of the sampling
c     prd:  a periodicity for calculating the rayleigh probability
c     is:  number for smoothing of periodgram
c     ipl:  if ipl=0 line printer only, otherwise plotter workes.
c     fmt:  format for reading point process data
c     t:  length of observed time interval of events
c     n:  number of events of data set
c     z(i):  data of events
c     nt:  number of particular periodcities to be investigated
c     tmpr(i):  particular periodcities to be investigated among others
c
      implicit real * 8 (a-h,o-z)
c     dimension z(10000),fmt(10)
      dimension z(10000)
      character*8 fmt(10)
c     dimension amx(10000),amg(10000)
      dimension xx(10000),tmpr(100)
c
      open(4,file='ptspec.init')
      read(4,*) nh,tmp,prd
      write(6,30) nh,tmp,prd
   30 format(1h ,' nh =',i5,5x,'tmp =',d15.6,5x,'prd =',d15.6)
      pi=3.1415926536d 00
      pi2=pi*2.d 00
      f=1.d0/tmp
      om=pi2*f
      read(4,*) is
      write(6,31) is
   31 format(1h ,' is =',i5)
    1 format(16i5)
      read(4,*) ipl
      read(4,*) in
      write(6,32) ipl
   32 format(1h ,' ipl =',i5,'    in =',i5)
c     read(4,3) fmt
    3 format(10a8)
c     write(6,33) fmt
   33 format(1h ,' fmt = ',10a8)
      open(in,file='work.data')
c     read(in,2) t,n
      read(in,*) t,n
    2 format(f10.4,i10)
c     read(in,fmt) (z(i),i=1,n)
      read(in,*) (z(i),i=1,n)
c     read(in,fmt) (amg(i),i=1,n)
    4 format(8f10.0)
c     read(5,fmt) amx1,amx2
      nn=0
      do 10 i=1,n
c     if(amg(i).lt.amx1.or.amg(i).gt.amx2) go to 10
      nn=nn+1
      xx(nn)=z(i)
c     amx(nn)=amg(i)
   10 continue
      read(4,*) nt
      write(6,34) nt
   34 format(1h ,' nt =',i5)
      read(4,*) (tmpr(i),i=1,nt)
      write(6,35) (tmpr(i),i=1,nt)
   35 format(1h ,' tmprs '/(1h ,5d13.5))
      write(6,22) om,nh
      write(6,21) is
      rpt=om/nh
      nh1=nh+1
      write(6,7) nn
      write(6,24) (xx(i),i=1,nn)
      amean=n/t
      write(6,27) tmp
      write(6,9) amean
      write(6,8) t
6     format(8f10.4)
7     format(1h ,' total number of occurrences,n=',i5/)
8     format(1h ,' length of obserbed interval =',d20.10)
9     format(1h ,' average=n/t =',d20.10)
21    format(1h ,i5,1x,'point smoothing')
22    format(1h ,' maximum frequency=',d18.10,3x,'divided into',i5,2x,
     *  'points')
23    format(i5,2d20.10)
24    format(1h ,5f12.4)
27    format(1h ,' minperod=',d20.10)
      return
      end
      subroutine cycle(xx,nn,prd)
      implicit real*8(a-h,o-z)
      dimension xx(10000)
      rwx=0.0
      rwy=0.0
      pai=3.14159265358979d0
      do 10 i=1,nn
      rwy=rwy+sin(2.d0*pai*xx(i)/prd)
      rwx=rwx+cos(2.d0*pai*xx(i)/prd)
   10 continue
      r2=rwx**2+rwy**2
      r1=sqrt(r2)
      phs=acos(rwx/r1)
      phs=phs/2.d0/pai*360.d0
      prb=0.0
      if(r2/nn.lt.100.d0) prb=exp(-r2/nn)
      write(6,1) prb,r1,rwx,rwy,phs
    1 format(1h ,' rayleigh prob =',d13.5,'   distance =',d13.5,
     &           '    rwx = ',d13.5,'     rwy = ',d13.5,
     &           '    phs = ',d13.5)
      return
      end
      subroutine period(h,g,w,n,t,nh1,ht,gt,wt,nt,rpt,t0,pi2,tmpr)
      implicit real*8 (a-h,o-z)
      dimension t(5000)
      dimension h(5000),g(5000)
      dimension w(5000)
      dimension tmpr(100),wt(200),ht(200),gt(200)
      do 120 i=1,nh1
      om=(i-1)*rpt
      w(i)=(i-1)*rpt
      a=0.d 00
      b=0.d 00
      do 110 j=1,n
      a=a+dcos(t(j)*om)
      b=b+dsin(t(j)*om)
110   continue
      h(i)=(a*a+b*b)/t0
      g(i)=h(i)/pi2
      ram=n/t0
      g(i)=g(i)/(ram/pi2)
      h(i)=dlog10(h(i))*10.d 00
120   continue
      do 1200 i=1,nt
      om=pi2/tmpr(i)
      wt(i)=om
      a=0.d 00
      b=0.d 00
      do 1100 j=1,n
      a=a+dcos(t(j)*om)
      b=b+dsin(t(j)*om)
1100  continue
      ht(i)=(a*a+b*b)/t0
      gt(i)=ht(i)/pi2
      ram=n/t0
      gt(i)=gt(i)/(ram/pi2)
      ht(i)=dlog10(ht(i))*10.d 00
1200  continue
      return
      end
      subroutine printr(n,x,y,z,nt,wt,ht,t0,tmp,isw1)
      real * 8 x,y,z,wt,ht,t0,tmp
      character*1 bl,pl,xo,st,ss(100)
      dimension x(1),y(1),z(1),wt(1),ht(1)
      data bl,pl/' ','|'/
      data xo,st/'o','*'/
      pi=3.14159265358979d0
      nn=n
      ymin=0.0
      ymax=-1.e10
*     do 10 i=1,nn
      do 10 i=2,nn
      if(ymin.gt.y(i)) ymin=y(i)
      if(ymax.lt.y(i)) ymax=y(i)
   10 continue
      x1=ymin
      x2=ymin+(ymax-ymin)/4
      x3=ymin+(ymax-ymin)/2
      x4=ymin+(ymax-ymin)*3/4
      x5=ymax
      write(6,1) x1,x3,x5
    1 format(1h /1h ,'  freq.     d.b.  ',e12.5,2(12x,e12.5)/1h ,21x,
     &       '+',4('------------+'))
      open(7,file='out.ptspec')
      ram=n/t0
      sgm=log(t0/tmp/pi)-log(0.05)
      sgm1=log10(ram*sgm)*10.d0
      sgm=log(t0/tmp/pi)-log(0.1)
      sgm2=log10(ram*sgm)*10.d0
      sgm=log(t0/tmp/pi)-log(0.01)
      sgm3=log10(ram*sgm)*10.d0
      write(7,*) sgm1,sgm2,sgm3
      write(7,*) nt
      do 80 i=1,nt
      write(7,*) wt(i)/(2*pi),ht(i)
   80 continue
      do 20 i=1,nn
      do 21 j=1,50
      ss(j)=bl
   21 continue
      l=(y(i)-ymin)*50/(ymax-ymin)+0.5
      if(nt.eq.0) go to 50
      do 30 j=1,nt
      if(abs(x(i)-wt(j)).lt.0.001) go to 40
   30 continue
   50 continue
c     if(l.gt.0) write(6,2) x(i),y(i),(xo,j=1,l)
c     if(l.le.0) write(6,2) x(i),y(i)
      do 11 j=1,l
      ss(j)=xo
   11 continue
      go to 25
   40 continue
c     if(l.gt.0) write(6,2) x(i),y(i),(st,j=1,l)
c     if(l.le.0) write(6,2) x(i),y(i)
      do 12 j=1,l
      ss(j)=st
   12 continue
    2 format(d10.3,d11.3,' i',51a1)
   25 continue
      ram=n/t0
      sgm=log(t0/tmp/pi)-log(0.05)
      sgm=log10(ram*sgm)*10.d0
      l=(sgm-ymin)*100/(ymax-ymin)+0.5
c     if(l.gt.1) write(6,3) (bl,j=1,l-1),pl
c     if(l.eq.1) write(6,3) pl
      ss(l)=pl
    3 format(1h+,26x,100a1)
      sgm=log(t0/tmp/pi)-log(0.1)
      sgm=log10(ram*sgm)*10.d0
      l=(sgm-ymin)*100/(ymax-ymin)+0.5
c     if(l.gt.1) write(6,3) (bl,j=1,l-1),pl
c     if(l.eq.1) write(6,3) pl
      ss(l)=pl
      sgm=log(t0/tmp/pi)-log(0.01)
      sgm=log10(ram*sgm)*10.d0
      l=(sgm-ymin)*50/(ymax-ymin)+0.5
c     if(l.gt.1) write(6,3) (bl,j=1,l-1),pl
c     if(l.eq.1) write(6,3) pl
      ss(l)=pl
c
      if(i.eq.1) go to 20
c
      write(6,2) x(i),y(i),(ss(j),j=1,50)
      write(7,*) x(i)/(2*pi),y(i)
   20 continue
      close(7)
      if(isw1.ne.1) return
      write(6,4)
      do 90 i=1,n
      xi=x(i)/(2*pi)
      yi=z(i)
      if(xi.lt.4./t0.or.yi.lt.4.) go to 90
      g=1./xi
      write(6,5) xi,g,yi
    5 format(1h ,3d15.5)
   90 continue
    4 format(1h /1h ,' list of high powers(>4) with period < t/4'//
     &           1h ,'     freqs.        periods          powers'/)
      return
      end
      subroutine smooth(s,h,nh1,is)
      implicit real*8 (a-h,o-z)
      dimension h(5000),s(5000)
      is1=2*is-1
      do 140 i=1,nh1
      ij=0
      s(i)=0.0
      do 130 j=1,is1
      ij1=i-is+j-1
      if(ij1.lt.1.or.ij1.gt.nh1) go to 130
      ij=ij+1
      s(i)=s(i)+h(ij1)
130   continue
      s(i)=s(i)/ij
140   continue
      return
      end
